@@ -11,8 +11,11 @@ coveralls-lcov -v -n coverage.filtered.info > coverage.c.json
 
 coverage report
 pip install codecov
-pip install coveralls-merge
-coveralls-merge coverage.c.json
+if [[ $TRAVIS_PYTHON_VERSION != "2.7_with_system_site_packages" ]]; then
+    # Not working here. Just skip it, it's being removed soon.
+    pip install coveralls-merge
+    coveralls-merge coverage.c.json
+fi
 codecov
 
 if [ "$TRAVIS_PYTHON_VERSION" == "2.7" ] && [ "$DOCKER" == "" ]; then
