@@ -1,12 +1,11 @@
-from .helper import PillowTestCase
+from PIL import CurImagePlugin, Image
 
-from PIL import Image, CurImagePlugin
+from .helper import PillowTestCase
 
 TEST_FILE = "Tests/images/deerstalker.cur"
 
 
 class TestFileCur(PillowTestCase):
-
     def test_sanity(self):
         im = Image.open(TEST_FILE)
 
@@ -20,8 +19,7 @@ class TestFileCur(PillowTestCase):
     def test_invalid_file(self):
         invalid_file = "Tests/images/flower.jpg"
 
-        self.assertRaises(SyntaxError,
-                          CurImagePlugin.CurImageFile, invalid_file)
+        self.assertRaises(SyntaxError, CurImagePlugin.CurImageFile, invalid_file)
 
         no_cursors_file = "Tests/images/no_cursors.cur"
 
