@@ -1,13 +1,10 @@
-from .helper import unittest, PillowTestCase, hopper
-from .helper import djpeg_available, cjpeg_available
-
-from io import BytesIO
 import os
 import sys
+from io import BytesIO
 
-from PIL import Image
-from PIL import ImageFile
-from PIL import JpegImagePlugin
+from PIL import Image, ImageFile, JpegImagePlugin
+
+from .helper import PillowTestCase, cjpeg_available, djpeg_available, hopper, unittest
 
 codecs = dir(Image.core)
 
@@ -372,6 +369,10 @@ class TestFileJpeg(PillowTestCase):
         with self.assertRaises(IOError):
             im.load()
 
+        # Test that the error is raised if loaded a second time
+        with self.assertRaises(IOError):
+            im.load()
+
     def _n_qtables_helper(self, n, test_file):
         im = Image.open(test_file)
         f = self.tempfile("temp.jpg")
