@@ -1,10 +1,8 @@
-from .helper import unittest, PillowTestCase, hopper, fromstring, tostring
-
 from io import BytesIO
 
-from PIL import Image
-from PIL import ImageFile
-from PIL import EpsImagePlugin
+from PIL import EpsImagePlugin, Image, ImageFile
+
+from .helper import PillowTestCase, fromstring, hopper, tostring, unittest
 
 try:
     from PIL import _webp
@@ -105,6 +103,14 @@ class TestImageFile(PillowTestCase):
             parser = ImageFile.Parser()
             parser.feed(1)
 
+    def test_negative_stride(self):
+        with open("Tests/images/raw_negative_stride.bin", "rb") as f:
+            input = f.read()
+        p = ImageFile.Parser()
+        p.feed(input)
+        with self.assertRaises(IOError):
+            p.close()
+
     def test_truncated_with_errors(self):
         if "zip_encoder" not in codecs:
             self.skipTest("PNG (zlib) encoder not available")
@@ -113,6 +119,10 @@ class TestImageFile(PillowTestCase):
         with self.assertRaises(IOError):
             im.load()
 
+        # Test that the error is raised if loaded a second time
+        with self.assertRaises(IOError):
+            im.load()
+
     def test_truncated_without_errors(self):
         if "zip_encoder" not in codecs:
             self.skipTest("PNG (zlib) encoder not available")
@@ -323,3 +333,13 @@ class TestPyDecoder(PillowTestCase):
         self.assertEqual(
             exif.get_ifd(0xA005), {1: "R98", 2: b"0100", 4097: 2272, 4098: 1704}
         )
+
+    def test_exif_shared(self):
+        im = Image.open("Tests/images/exif.png")
+        exif = im.getexif()
+        self.assertIs(im.getexif(), exif)
+
+    def test_exif_str(self):
+        im = Image.open("Tests/images/exif.png")
+        exif = im.getexif()
+        self.assertEqual(str(exif), "{274: 1}")
